(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.FetchEventSource = factory());
}(this, (function () { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function _classPrivateFieldGet(receiver, privateMap) {
    var descriptor = privateMap.get(receiver);

    if (!descriptor) {
      throw new TypeError("attempted to get private field on non-instance");
    }

    if (descriptor.get) {
      return descriptor.get.call(receiver);
    }

    return descriptor.value;
  }

  function _classPrivateFieldSet(receiver, privateMap, value) {
    var descriptor = privateMap.get(receiver);

    if (!descriptor) {
      throw new TypeError("attempted to set private field on non-instance");
    }

    if (descriptor.set) {
      descriptor.set.call(receiver, value);
    } else {
      if (!descriptor.writable) {
        throw new TypeError("attempted to set read only private field");
      }

      descriptor.value = value;
    }

    return value;
  }

  var isAbsoluteUrl = url => {
    if (typeof url !== 'string') {
      throw new TypeError(`Expected a \`string\`, got \`${typeof url}\``);
    } // Don't match Windows paths `c:\`


    if (/^[a-zA-Z]:\\/.test(url)) {
      return false;
    } // Scheme: https://tools.ietf.org/html/rfc3986#section-3.1
    // Absolute URL: https://tools.ietf.org/html/rfc3986#section-4.3


    return /^[a-zA-Z][a-zA-Z\d+\-.]*:/.test(url);
  };

  /* eslint-disable no-underscore-dangle */

  /* eslint-disable no-await-in-loop */
  // ready state
  const CONNECTING = 0;
  const OPEN = 1;
  const CLOSED = 2;

  var _controller = new WeakMap();

  var _config = new WeakMap();

  var _lastId = new WeakMap();

  class BaseFetchEventSource extends EventTarget {
    constructor(url, config) {
      super(); // check url is set

      _defineProperty(this, "url", void 0);

      _defineProperty(this, "withCredentials", void 0);

      _defineProperty(this, "readyState", CONNECTING);

      _defineProperty(this, "onopen", void 0);

      _defineProperty(this, "onmessage", void 0);

      _defineProperty(this, "onerror", void 0);

      _defineProperty(this, "_fetch", void 0);

      _defineProperty(this, "_decode", void 0);

      _defineProperty(this, "_getOrigin", void 0);

      _defineProperty(this, "_transformStream", stream => stream);

      _controller.set(this, {
        writable: true,
        value: new AbortController()
      });

      _config.set(this, {
        writable: true,
        value: {
          headers: new Headers(),
          autoStart: true,
          reconnectionDelay: 2000,
          withCredentials: false
        }
      });

      _lastId.set(this, {
        writable: true,
        value: ''
      });

      if (!url) {
        throw new Error('Cannot open an FetchEventSource to an empty URL.');
      }

      this.url = url;

      _classPrivateFieldSet(this, _config, { ..._classPrivateFieldGet(this, _config),
        ...config
      });

      this.withCredentials = _classPrivateFieldGet(this, _config).withCredentials; // if autoStart set then start connection

      if (_classPrivateFieldGet(this, _config).autoStart) {
        // start on nextTick so everything is initialized
        setTimeout(this.start.bind(this), 0);
      }
    }

    async start() {
      try {
        await this._connect();
      } catch (e) {
        // start new connection if user did not abort
        if (!_classPrivateFieldGet(this, _controller).signal.aborted) {
          this._error(e);

          await new Promise(resolve => setTimeout(resolve, _classPrivateFieldGet(this, _config).reconnectionDelay));
          this.start();
        }
      }
    }

    async _connect() {
      _classPrivateFieldSet(this, _controller, new AbortController()); // setup fetch and headers


      const {
        fetch = this._fetch,
        headers
      } = _classPrivateFieldGet(this, _config);

      headers.set('Accept', 'text/event-stream');

      if (_classPrivateFieldGet(this, _lastId)) {
        headers.set('Last-Event-ID', _classPrivateFieldGet(this, _lastId));
      } // connecting


      this.readyState = CONNECTING; // make fetch request to SSE

      const response = await fetch(this.url, {
        signal: _classPrivateFieldGet(this, _controller).signal,
        headers,
        credentials: _classPrivateFieldGet(this, _config).withCredentials ? 'include' : 'same-origin',
        cache: 'no-store'
      }); // fail if not 200 response

      if (!response || response.status !== 200) {
        this.close();

        this._error(new Error('Bad status'));

        return;
      } // check is correct content-type from response


      const type = response.headers.get('content-type');

      if (!type || type.split(';')[0] !== 'text/event-stream') {
        this.close();

        this._error(new Error('Bad content-type'));

        return;
      } // connected


      this.readyState = OPEN;
      this.dispatchEvent(new Event('open'));
      if (this.onopen) this.onopen(); // transform stream and process reader

      const reader = this._transformStream(response.body).getReader();

      await this._process(reader);
    }

    async _process(reader) {
      let type = '';
      let data = [];
      let buffer = '';

      for (;;) {
        // stop if has been aborted
        if (_classPrivateFieldGet(this, _controller).signal.aborted) {
          reader.cancel();
          return;
        } // check if buffer contains CRLF, LF or CR


        const regex = /\r\n|\r|\n/.exec(buffer);

        if (regex) {
          // get next line which can be a new line
          const line = buffer.slice(0, regex.index) || regex[0];
          buffer = buffer.slice(regex.index + regex[0].length);
          const [key, value = ''] = line.split(/:(.+)?/, 2).map(v2 => v2 === null || v2 === void 0 ? void 0 : v2.trimLeft()); // send event if line is a new line

          if (line.match(/^(\r\n|\r|\n)$/) && (type || data)) {
            // create event
            const event = new MessageEvent(type, {
              origin: this._getOrigin(),
              data: data.join('\n'),
              lastEventId: _classPrivateFieldGet(this, _lastId)
            }); // dispatch event

            this.dispatchEvent(event);
            if (type === 'message' && this.onmessage) this.onmessage(event); // reset event data

            type = '';
            data = [];
          } // process line


          switch (key) {
            case 'event':
              {
                type = value;
                break;
              }

            case 'data':
              {
                type = type || 'message';
                data.push(value);
                break;
              }

            case 'id':
              {
                _classPrivateFieldSet(this, _lastId, value);

                break;
              }

            case 'retry':
              {
                const num = Number(value);
                if (num) _classPrivateFieldGet(this, _config).reconnectionDelay = num;
                break;
              }
          }
        } else {
          // read next data
          const {
            value: byteValue,
            done
          } = await reader.read();
          if (done) throw Error('Connection terminated'); // decode value for both browsers and node

          const value = typeof byteValue === 'string' ? byteValue : this._decode(byteValue);
          buffer += value;
        }
      }
    }

    close() {
      this.readyState = CLOSED;

      _classPrivateFieldGet(this, _controller).abort();
    }

    _error(error) {
      const event = new ErrorEvent('error', {
        error,
        message: error.message
      });
      this.dispatchEvent(event);
      if (this.onerror) this.onerror(event);
    }

  }

  const decoder = new TextDecoder();
  class FetchEventSource extends BaseFetchEventSource {
    constructor(...args) {
      super(...args);

      _defineProperty(this, "_fetch", window.fetch.bind(window));

      _defineProperty(this, "_decode", bytes => decoder.decode(bytes));

      _defineProperty(this, "_getOrigin", () => {
        if (isAbsoluteUrl(this.url)) {
          return new URL(this.url).origin;
        }

        return window.location.origin;
      });
    }

  }

  return FetchEventSource;

})));
//# sourceMappingURL=index.js.map
